from aiohttp import web
from aiohug import swagger, RouteTableDef
from marshmallow import Schema, fields


class TestingFieldsSchema(Schema):
    integer = fields.Integer()
    float = fields.Float()
    boolean = fields.Boolean()
    datetime = fields.DateTime()
    timedelta = fields.TimeDelta()
    dictionary = fields.Dict()
    url = fields.Url()
    email = fields.Email()


class TestingSchema(Schema):
    hug_types_number = fields.Integer()
    hug_types_greater_than_5 = fields.Integer()
    hug_types_in_range_1_5 = fields.Integer()


routes = RouteTableDef()


@routes.get("/hello/{name}/")
async def hello(name: fields.String(), greeting: fields.String() = "Hello"):
    return {"msg": f"{greeting}, {name}"}


@routes.get("/swagger/hug/{hug_types_number}/{hug_types_greater_than_5}/")
async def openapi_test(request, hug_timer, hug_types_number: fields.Number()):
    """Endpoint with hug.types

    Not versioned api method"""
    return {"hug_types_number": hug_types_number}


@routes.get("/swagger/marshmallow/{swagger_types_number}/", versions=[2, 3])
@swagger.response(200, description="Good response", schema=TestingSchema)
@swagger.response(400, description="Bad response")
async def openapi_test_swagger_types(
    request,
    hug_timer,
    swagger_types_number: fields.Integer(),
    swagger_types_number_in_query: fields.Integer() = 3,
) -> TestingSchema():
    """Endpoint with marshmallow types"""
    return {
        "swagger_types_number": swagger_types_number,
        "swagger_types_number_in_query": swagger_types_number_in_query,
    }


@routes.post("/swagger/marshmallow/post-body")  # TODO: check with last slash
@swagger.response(201, description="Created", schema=TestingSchema())
async def openapi_post_body(body: TestingSchema()) -> TestingSchema():
    return body


app = web.Application()
app.add_routes(routes)



import os
import yaml
from aiohttp import web

from aiohug import RouteTableDef
from aiohug import swagger

routes = RouteTableDef()


@swagger.spec(exclude=True)
@routes.get("/swagger.json")
async def swagger_json(request):
    return swagger.generate_swagger(request.app)


@swagger.spec(exclude=True)
@routes.get("/swagger.yaml")
async def swagger_yaml(request):
    return web.Response(
        text=yaml.dump(swagger.generate_swagger(request.app)), content_type="text/yaml"
    )


@swagger.spec(exclude=True)
@routes.get("/swagger/", name="swagger")
async def root():
    template_path = os.path.join(
        os.path.dirname(os.path.abspath(__file__)), "templates", "index.html"
    )

    with open(template_path) as template:
        return web.Response(
            text=template.read().replace("{{ swagger_url }}", "/swagger.json"),
            content_type="text/html",
        )



import pytest

from aiohttp import web
from aiohug import RouteTableDef
from ..handlers import routes as swagger_handlers


@pytest.fixture
def app() -> web.Application:
    routes = RouteTableDef()

    @routes.get("/")
    async def ping():
        return "pong"

    app = web.Application()
    app.add_routes(routes)
    app.add_routes(swagger_handlers)
    return app


async def test_swagger_json(app, test_client):
    client = await test_client(app)
    resp = await client.get("/swagger.json")
    body = await resp.json()
    assert resp.status == 200
    assert resp.content_type == 'application/json'

    # assert body == {}


async def test_swagger_yaml(app, test_client):
    client = await test_client(app)
    resp = await client.get("/swagger.yaml")
    assert resp.status == 200
    assert resp.content_type == 'text/yaml'
    text = await resp.text()

    # assert text == ''


async def test_swagger(app, test_client):
    client = await test_client(app)
    resp = await client.get("/swagger/")
    assert resp.status == 200
    assert resp.content_type == 'text/html'

    # text = await resp.text()
